/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * AppExplorerWindow.java: The main ui window and entry point for the app
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import app.appexplorer.file.IBrwFile;
import app.appexplorer.file.IniFile;
import app.appexplorer.file.ListedFile;
import app.appexplorer.hunt.BrwFileHunter;
import app.appexplorer.hunt.FolderSearch;
import app.appexplorer.hunt.SetupFileHunter;
import app.appexplorer.log.ComponentLogger;
import app.appexplorer.log.ILoggerComponent;
import app.appexplorer.util.GlobalUtils;

/**
 * This class defines the main UI window of the app. It also has the
 * applications only entry point main().
 * 
 * @author ali
 * @version 2
 */
public final class AppExplorerWindow extends JFrame implements
        ILoggerComponent, TreeSelectionListener, FocusListener {

    /**
     * Handles events for all UI components for this window.
     * 
     * @version 2
     * @author ali
     */
    private final class ActionEventsHandler implements ActionListener {
        //File Menu
        public final static String EXIT = "Exit";

        public final static String OPENFIL = "Open File";

        public final static String OPENFOL = "Open Folder";

        public final static String SCANFOL = "Scan Folder";

        public final static String SCANDRV = "Scan Drive";

        //Edit Menu
        public final static String COPY = "Copy Text";

        //View Menu
        public final static String SHOWTOOLBAR = "Show Toolbar";

        public final static String SHOWDESC = "Show Description";

        public final static String TABONTOP = "Show Tabs on Top";

        //Help Menu
        public final static String ABOUT = "About";

        //Log Menu
        public final static String LOGOFF = "Log OFF";

        public final static String LOGALL = "Log ALL";

        public final static String LOGINFO = "Log info and worse";

        public final static String LOGWARN = "Log warnings and worse";

        public final static String LOGERROR = "Log errors and worse";

        public final static String LOGDEBUG = "Debug Mode";

        public final static String CLEAR = "Clear";

        public ActionEventsHandler() {
            super();
        }

        /*
         * Handles Menu and button events
         * 
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent ae) {
            if (EXIT.equalsIgnoreCase(ae.getActionCommand())) {
                eventExit();
            }

            if (OPENFIL.equalsIgnoreCase(ae.getActionCommand())) {
                eventOpenFile();
            }

            if (OPENFOL.equalsIgnoreCase(ae.getActionCommand())) {
                eventOpenFol();
            }

            if (SCANFOL.equalsIgnoreCase(ae.getActionCommand())) {
                eventScanFol();
            }

            if (SCANDRV.equalsIgnoreCase(ae.getActionCommand())) {
                eventScanDrive();
            }

            if (COPY.equalsIgnoreCase(ae.getActionCommand())) {
                eventCopy();
            }

            if (CLEAR.equalsIgnoreCase(ae.getActionCommand())) {
                eventClearLog();
            }

            if (LOGALL.equalsIgnoreCase(ae.getActionCommand())) {
                eventChangeLogLevel(Level.ALL);
            }
            if (LOGOFF.equalsIgnoreCase(ae.getActionCommand())) {
                eventChangeLogLevel(Level.OFF);
            }
            if (LOGINFO.equalsIgnoreCase(ae.getActionCommand())) {
                eventChangeLogLevel(Level.INFO);
            }
            if (LOGWARN.equalsIgnoreCase(ae.getActionCommand())) {
                eventChangeLogLevel(Level.WARNING);
            }
            if (LOGERROR.equalsIgnoreCase(ae.getActionCommand())) {
                eventChangeLogLevel(Level.SEVERE);
            }
            if (LOGDEBUG.equalsIgnoreCase(ae.getActionCommand())) {
                eventChangeLogLevel(Level.FINEST);
            }
            if (ABOUT.equalsIgnoreCase(ae.getActionCommand())) {
                eventAbout();
            }
            if (SHOWDESC.equalsIgnoreCase(ae.getActionCommand())) {
                eventShowDesc();
            }
            if (SHOWTOOLBAR.equalsIgnoreCase(ae.getActionCommand())) {
                eventToggleToolbar(ae.getSource());
            }
            if (TABONTOP.equalsIgnoreCase(ae.getActionCommand())) {
                eventTabsOnTop(ae.getSource());
            }
        }

        public void eventClearLog() {
            m_taLogBox.setText("");
        }

        public void eventAbout() {
            new AboutDialog(AppExplorerWindow.m_wndMain);
        }

        public void eventShowDesc() {
            m_tabs.setSelectedIndex(m_tabs.indexOfTab("Description"));
        }

        public void eventToggleToolbar(Object cbObject) {
            JCheckBoxMenuItem cb = (JCheckBoxMenuItem) cbObject;
            m_toolbar.setVisible(cb.isSelected());
        }

        public void eventTabsOnTop(Object cbObject) {
            JCheckBoxMenuItem cb = (JCheckBoxMenuItem) cbObject;
            eventTabsOnTop(cb.isSelected());
        }

        public void eventTabsOnTop(boolean bOnTop) {
            if (bOnTop) {
                m_tabs.setTabPlacement(JTabbedPane.TOP);
            } else {
                m_tabs.setTabPlacement(JTabbedPane.BOTTOM);
            }
        }

        public void eventChangeLogLevel(Level level) {
            m_globalLogger.log(Level.INFO, "Logging level changed to "
                    + level.getName());
            m_globalLogger.setLevel(level);
        }

        /**
         * Copy text from the last focused TextArea. TODO: Extend this function
         * to copy text from any window.
         */
        public void eventCopy() {
            if (m_strSelected != null) {
                GlobalUtils.copyToSysClipboard(m_strSelected);
            }
        }

        /**
         * Quit the application
         */
        public void eventExit() {
            System.exit(0);
        }

        /**
         * Scans the entered folder path for applications.
         */
        public void eventScanFol() {
            //show a standard input dialog.
            String selVal = JOptionPane.showInputDialog(m_wndMain,
                    "Enter the folder path you wish to scan.", "Scan Folder",
                    JOptionPane.QUESTION_MESSAGE);

            if (selVal != null && selVal.length() > 0) {
                //check if the entered path represents an existing folder.
                File f = new File(selVal);
                if (!f.isDirectory()) {
                    JOptionPane.showMessageDialog(m_wndMain, selVal
                            + " is not a valid folder path",
                            "Scan Folder - Error", JOptionPane.ERROR_MESSAGE);
                } else {
                    //ok.. all looks good, go ahead and search
                    m_wndMain.populate(selVal);
                }
            }
        }

        /**
         * Scans the selected drive root for applications. Note that internally
         * there is no difference in scanning drives or folders, the user can
         * choose select folder and enter a drive, and it should work fine. This
         * is just for added convenience.
         */
        public void eventScanDrive() {
            //show a list of filesystem roots
            File[] roots = File.listRoots();
            if (roots.length != 0) {
                File selVal = (File) JOptionPane.showInputDialog(m_wndMain,
                        "Select the drive you wish to scan.", "Scan Drive",
                        JOptionPane.QUESTION_MESSAGE, null, roots, roots[0]);

                if (selVal != null) {
                    if (!selVal.isDirectory()) {
                        //this can happen if the selected root represents a
                        // removable drive that is not currently mounted.
                        JOptionPane
                                .showMessageDialog(m_wndMain, selVal
                                        + " could not be scanned",
                                        "Scan Drive - Error",
                                        JOptionPane.ERROR_MESSAGE);
                    } else {
                        //ok.. all looks good, go ahead and search
                        m_wndMain.populate(selVal.toString());
                    }

                }
            } else {
                //No drives/roots could be found on this os, strange!
                JOptionPane.showMessageDialog(m_wndMain,
                        "No drives could be found.", "Scan Drive - Error",
                        JOptionPane.ERROR_MESSAGE);
            }
        }

        /**
         * This function checks if a file is selected and then tries and open
         * the selected file.
         * 
         * If it is unable to open the file, it tries to open the containing
         * folder. If that also fails it copies the files complete path to the
         * users clipboard so that the user may manually open the file, and
         * displays an error message.
         */
        public void eventOpenFile() {
            ListedFile lf = (ListedFile) m_list.getSelectedValue();
            if (lf != null) {
                if (!GlobalUtils.os.openFile(lf.getPath())) {
                    //something went wrong
                    //Make the error message string
                    String msg = "Failed to automatically open file: \n\t"
                            + GlobalUtils.toCanonicalPath(lf.getPath());

                    //log the message
                    m_globalLogger.log(Level.SEVERE, msg);

                    /*
                     * lets be smart, instead of throwing an error to the user
                     * try opening the containing folder for this file
                     */
                    if (!GlobalUtils.os.openFolder(lf.getPath())) {
                        //open folder failed too! we have no choice but to show
                        // up an error.
                        m_globalLogger
                                .log(Level.SEVERE,
                                        "Failed to automatically open containing folder");

                        //copy the files path to clipboard
                        GlobalUtils.copyToSysClipboard(GlobalUtils
                                .toCanonicalPath(lf.getPath()));

                        //show the error message to the user
                        JOptionPane
                                .showMessageDialog(
                                        m_wndMain,
                                        msg
                                                + "\nThe path of this file has been copied to your clipboard",
                                        "Error", JOptionPane.ERROR_MESSAGE);
                    }
                }
            } else/* lf==null */{
                //No file is selected from the list
                JOptionPane
                        .showMessageDialog(
                                m_wndMain,
                                "No file is selected. Select a file from the right side pane.",
                                "Open File - Error",
                                JOptionPane.INFORMATION_MESSAGE);
            }
        }

        /**
         * This function checks if a application is selected and then tries and
         * open the folder containing the application.
         * 
         * If it fails it copies the folders complete path to the users
         * clipboard so that the user may manually open the folder, and displays
         * an error message.
         */
        public void eventOpenFol() {
            if (m_currAppDesc != null) {
                if (!GlobalUtils.os.openFolder(m_currAppDesc
                        .getContainingFolder())) {
                    //something went wrong
                    //Make the error message string
                    String msg = "Failed to automatically open folder: \n\t"
                            + GlobalUtils.toCanonicalPath(m_currAppDesc
                                    .getContainingFolder());

                    //log the message
                    m_globalLogger.log(Level.SEVERE, msg);

                    //copy the folders path to clipboard
                    GlobalUtils.copyToSysClipboard(GlobalUtils
                            .toCanonicalPath(m_currAppDesc
                                    .getContainingFolder()));

                    //show error box to user
                    JOptionPane
                            .showMessageDialog(
                                    m_wndMain,
                                    msg
                                            + "\nThe path of this folder has been copied to your clipboard",
                                    "Error", JOptionPane.ERROR_MESSAGE);
                }
            } else /* currentFile == null */{
                //the user has not selected any application
                JOptionPane
                        .showMessageDialog(
                                m_wndMain,
                                "No application is selected. Select an application from the left side pane.",
                                "Error", JOptionPane.INFORMATION_MESSAGE);
            }
        }
    }

    /**
     * Handles click events on the listbox.
     * 
     * @version 2
     * @author ali
     */
    private final class ListClickListner extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {

            ListedFile lf = (ListedFile) m_list.getSelectedValue();
            //check if something is selected
            if (lf != null) {
                //Enable the selected files path to be copied to the clipboard.
                m_strSelected = new StringSelection(GlobalUtils
                        .toCanonicalPath(lf.getPath()));
            }

            //handle a right click event
            if (e.isPopupTrigger()) {
                //TODO show a pop-up menu with various useful options
                return;
            }

            //handle double click on a selected file.
            if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2) {
                m_aeHandler.eventOpenFile();
            }
        }
    }

    /**
     * Utility class, defines constants used to read/save the applications
     * properties.
     * 
     * @version 2
     * @author ali
     */
    public final class AppPropConstants {
        public final static String sStartFolder = "app.startfolder";

        public final static String sLogLevel = "app.loglevel";

        public final static String sAppTitle = "app.title";

        public final static String sLogOptions = "app.logoptions";

        public final static String sLookAndFeel = "app.lookandfeel";

        public final static String sWndPosX = "app.ui.x";

        public final static String sWndPosY = "app.ui.y";

        public final static String sWndHeight = "app.ui.height";

        public final static String sWndWidth = "app.ui.width";

        public final static String sWndToolbar = "app.ui.toolbar";

        public final static String sTabsOnTop = "app.ui.tabsontop";
    }

    /**
     * Logger used for logging messages.
     */
    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /*
     * Data Members for the entire app - all will be init in main()
     */

    /**
     * This variable points to the applications main window.
     */
    private static AppExplorerWindow m_wndMain = null;

    /**
     * This variable points hashtable contining many of the applications
     * settings.
     */
    private static Hashtable m_htAppConfig = null;

    /*
     * Data Members for the Main Window - most will be init by the constructor
     * AppExplorerWindow()
     */

    /**
     * The handler used for handling all UI events (menu, toolbar etc).
     */
    private ActionEventsHandler m_aeHandler = null;

    /**
     * The horizontal Split Pane
     */
    private JSplitPane m_splitHz = null;

    /**
     * The vertical Split Pane
     */
    private JSplitPane m_splitVt = null;

    /**
     * The short description area.
     */
    private JTextArea m_taDescription = null;

    /**
     * The long description area
     */
    private JTextArea m_taDescriptionFull = null;

    /**
     * The category tree
     */
    private JTree m_tree = null;

    /**
     * The setup files list
     */
    private JList m_list = null;

    /**
     * The main menu bar
     */
    private JMenuBar m_mainMenu = null;

    /**
     * The tabbed pane
     */
    private JTabbedPane m_tabs = null;

    /**
     * The text area used for logging
     */
    private JTextArea m_taLogBox = null;

    /**
     * The main toolbar
     */
    private JToolBar m_toolbar = null;

    /**
     * Points to the text in currently focused window. May be null.
     */
    private StringSelection m_strSelected = null;

    /**
     * Points to the currently selected "application descriptor". May be null.
     */
    private IBrwFile m_currAppDesc = null;

    /**
     * Starts up the application by scanning for "application descriptors" and
     * initializing the main window.
     * 
     * @param args -
     *            The first param args[0] can be a string specifying the
     *            starting path location for searching applications.
     */
    public static void main(String[] args) {
        String sStartingFolder = null;

        //see if the starting folder is given on the command line.
        if (args.length > 0 && args[0] != null && args[0].trim().length() > 0) {
            sStartingFolder = args[0].trim();
        } else {
            //use the system property
            sStartingFolder = System.getProperty("user.dir");
        }

        //read config info
        readConfig(sStartingFolder);

        //change the look and feel
        setLookAndFeel();

        //set logging level
        m_globalLogger.setLevel(GlobalUtils.nvlLogLevel(m_htAppConfig
                .get(AppPropConstants.sLogLevel), Level.INFO));

        //see if the application properties give us a different folder to start
        // scanning from.
        sStartingFolder = GlobalUtils.nvlString(m_htAppConfig
                .get(AppPropConstants.sStartFolder), sStartingFolder);

        //Create the one and only one main window
        m_wndMain = new AppExplorerWindow();
        //populate the views
        m_wndMain.populate(sStartingFolder);

        //Go!
        m_wndMain.setVisible(true);

        //do any UI adjustments if needed.
        m_wndMain.doAfterVisible();
    }

    /**
     * Do any UI component adjustments that cannot be done before actually
     * displaying the window.
     * 
     * This method is called from main after it displays the main window.
     */
    public void doAfterVisible() {
        //adjust the horizontal split
        m_splitHz.setDividerLocation(0.7);
        //adjust the vertical split
        m_splitVt.setDividerLocation(0.4);
    }

    /**
     * Set the LookAndFeel to use for the application. This is specified by the
     * app.lookandfeel property in autorun.inf <br>
     * Leave this blank to use the Java look and feel. <br>
     * Setting this value to "system" loads the OS specific look and feel. <br>
     * Setting this value to "motif" loads the Motif specific look and feel.
     * <br>
     * TODO Add support for Skin look and feel
     */
    private static void setLookAndFeel() {
        //read the look and feel
        String sLookAndFeel = GlobalUtils.nvlString(m_htAppConfig
                .get(AppPropConstants.sLookAndFeel), "");
        try {
            if (sLookAndFeel.length() != 0) {
                if ("system".equalsIgnoreCase(sLookAndFeel))
                    UIManager.setLookAndFeel(UIManager
                            .getSystemLookAndFeelClassName());
                if ("motif".equalsIgnoreCase(sLookAndFeel))
                    UIManager
                            .setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            }
        } catch (Exception e) {
        }
    }

    /**
     * Try to locate and read properties from autorun.inf
     * 
     * @param startingFolder -
     *            The folder to search for autorun.inf. Not null.
     */
    private static void readConfig(String sStartingFolder) {
        Hashtable props = null;

        //look for the autorun.inf, we do a search to avoid cAsE problems in
        // locating the name "autorun.inf"
        FolderSearch fs = new FolderSearch("autorun.inf", sStartingFolder);
        fs.doSearch();
        File autorun = fs.getFile();
        if (autorun != null) {
            //ok we found the file

            m_globalLogger.log(Level.INFO,
                    "Found autorun.inf, will use options set in this file.");

            IniFile ar = new IniFile(autorun);
            /*
             * Read our section from the file. If this section is missing the
             * returned hash will be empty, thats ok.
             */
            props = ar.readSection("appexp");
        } else {
            //Create an empty hash, which will default all options
            props = new Hashtable();
        }

        /*
         * From this point forward we try to read all properties from the
         * m_htAppConfig hash, using nvl functions from GlobalUtils and take the
         * default value whenever the property is found missing
         */
        m_htAppConfig = props;
    }

    /**
     * Constructs the main window for the app, initializes its components,
     * attaches any action handlers, and finally displays the window.
     */
    public AppExplorerWindow() {
        setDefaultCloseOperation(EXIT_ON_CLOSE);

        //set the windows title
        setTitle(GlobalUtils.nvlString(m_htAppConfig
                .get(AppPropConstants.sAppTitle), "Application Explorer"));

        //Set the windows size.
        setSize(GlobalUtils.nvlInt(m_htAppConfig
                .get(AppPropConstants.sWndWidth), 650), GlobalUtils.nvlInt(
                m_htAppConfig.get(AppPropConstants.sWndHeight), 450));

        //Set the windows location.
        setLocation(GlobalUtils.nvlInt(m_htAppConfig
                .get(AppPropConstants.sWndPosX), 10), GlobalUtils.nvlInt(
                m_htAppConfig.get(AppPropConstants.sWndPosY), 10));

        //Layout manager, we use BorderLayout to allow toolbar docking
        getContentPane().setLayout(new BorderLayout());

        //Create Handler
        m_aeHandler = new ActionEventsHandler();

        //Create and attach the main toolbar
        makeToolBar();

        //Create and attach the main MenuBar
        setJMenuBar(m_mainMenu = new JMenuBar());

        //Create and add menus to the main menubar
        m_mainMenu.add(makeFileMenu());
        m_mainMenu.add(makeEditMenu());
        m_mainMenu.add(makeViewMenu());
        //add log menu only if logoptions are enabled
        if (GlobalUtils.nvlBoolean(m_htAppConfig
                .get(AppPropConstants.sLogOptions), true)) {
            m_mainMenu.add(makeLogMenu());
        }
        m_mainMenu.add(makeHelpMenu());

        /*
         * Create and attach our Components
         */

        //The tabbed pane
        m_tabs = new JTabbedPane(JTabbedPane.BOTTOM);
        //Position of the tabs
        m_aeHandler.eventTabsOnTop(GlobalUtils.nvlBoolean(m_htAppConfig
                .get(AppPropConstants.sTabsOnTop), false));

        //Add the tab pane to the center
        getContentPane().add(m_tabs, BorderLayout.CENTER);

        //We treat horizontal as along the horizon
        m_splitHz = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        m_splitHz.setOneTouchExpandable(true);
        m_splitHz.setResizeWeight(1);
        //set the size, because on some look and feels the divider becomes 1
        // pixel only, hiding the OneTouchExpandable buttons
        m_splitHz.setDividerSize(10);

        //So this becomes vertical
        m_splitVt = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        m_splitVt.setResizeWeight(1);

        //the vertical split is the top component of the horizontal split,
        // giving us a 3-way split window
        m_splitHz.setTopComponent(m_splitVt);
        //the bottom component of the horizontal split is a short-description
        // box
        m_splitHz.setBottomComponent(new JScrollPane(
                m_taDescription = new JTextArea()));

        m_taDescription.setEditable(false);
        m_taDescription.setLineWrap(true);

        //the left component of the vertical split is the Tree
        m_splitVt.setLeftComponent(new JScrollPane(m_tree = new JTree(
                new DefaultTreeModel(new DefaultMutableTreeNode("")))));
        //while the right component is the filelist
        m_splitVt.setRightComponent(new JScrollPane(m_list = new JList()));
        m_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        //The first tab is this 3-way split browsing window
        m_tabs.addTab("Browse", m_splitHz);
        //the second tab is the long description.
        m_tabs.addTab("Description", new JScrollPane(
                m_taDescriptionFull = new JTextArea()));
        m_taDescriptionFull.setEditable(false);
        m_taDescriptionFull.setLineWrap(true);

        // the third tab is the Log.
        m_tabs.addTab("Log", new JScrollPane(m_taLogBox = new JTextArea()));
        m_taLogBox.setEditable(false);
        m_tabs.setEnabledAt(m_tabs.indexOfTab("Log"), GlobalUtils.nvlBoolean(
                m_htAppConfig.get(AppPropConstants.sLogOptions), true));

        //Add Listeners
        m_tree.addTreeSelectionListener(this);
        m_list.addMouseListener(new ListClickListner());
        m_taDescription.addFocusListener(this);
        m_taDescriptionFull.addFocusListener(this);
        m_taLogBox.addFocusListener(this);

        //Attach the main window to a ComponentLogger and add it as a handler
        // to the global logger.
        m_globalLogger.addHandler(new ComponentLogger(this));
    }

    /**
     * Creates and attaches the toolbar to the main window. Call only once
     * during init.
     */
    private void makeToolBar() {
        //Create Toolbar
        m_toolbar = new JToolBar("Main");

        getContentPane().add(m_toolbar, BorderLayout.NORTH);
        JButton b = null;

        m_toolbar.add(b = new JButton(ActionEventsHandler.SCANFOL,
                new ImageIcon(ClassLoader
                        .getSystemResource("icons/magnify.gif"))));
        b.setToolTipText("Scan a folder for applications");
        m_toolbar.addSeparator();
        b.addActionListener(m_aeHandler);

        m_toolbar.add(b = new JButton(ActionEventsHandler.OPENFIL,
                new ImageIcon(ClassLoader.getSystemResource("icons/app.gif"))));
        b.setToolTipText("Open/Execute the selected file");
        b.addActionListener(m_aeHandler);

        m_toolbar
                .add(b = new JButton(ActionEventsHandler.OPENFOL,
                        new ImageIcon(ClassLoader
                                .getSystemResource("icons/folder.gif"))));
        b.setToolTipText("Open the folder containing this application");
        b.addActionListener(m_aeHandler);
        m_toolbar.addSeparator();

        m_toolbar.add(b = new JButton(ActionEventsHandler.COPY, new ImageIcon(
                ClassLoader.getSystemResource("icons/copy.gif"))));
        b.setToolTipText("Copy text from the focused window to the clipboard");
        b.addActionListener(m_aeHandler);

        m_toolbar.addSeparator();
        m_toolbar.add(b = new JButton(ActionEventsHandler.ABOUT, new ImageIcon(
                ClassLoader.getSystemResource("icons/help.gif"))));
        b.setToolTipText("About this application");
        b.addActionListener(m_aeHandler);

        //Should we show the toolbar?
        m_toolbar.setVisible(GlobalUtils.nvlBoolean(m_htAppConfig
                .get(AppPropConstants.sWndToolbar), true));
    }

    /**
     * Creates and returns the File Sub menu. Call only once during init.
     * 
     * @return the created File Menu.
     */
    private JMenu makeFileMenu() {
        JMenuItem mi = null;

        JMenu menu = new JMenu("File");

        menu.add(mi = new JMenuItem(ActionEventsHandler.SCANDRV));
        mi.addActionListener(m_aeHandler);

        menu.add(mi = new JMenuItem(ActionEventsHandler.SCANFOL));
        mi.addActionListener(m_aeHandler);
        menu.add(new JSeparator());

        menu.add(mi = new JMenuItem(ActionEventsHandler.OPENFIL));
        mi.addActionListener(m_aeHandler);

        menu.add(mi = new JMenuItem(ActionEventsHandler.OPENFOL));
        mi.addActionListener(m_aeHandler);

        menu.add(new JSeparator());
        menu.add(mi = new JMenuItem(ActionEventsHandler.EXIT));
        mi.addActionListener(m_aeHandler);

        return menu;
    }

    /**
     * Creates and returns the Help Sub menu. Call only once during init.
     * 
     * @return the created Help Menu.
     */
    private JMenu makeHelpMenu() {
        JMenuItem mi = null;

        JMenu menu = new JMenu("Help");

        menu.add(mi = new JMenuItem(ActionEventsHandler.ABOUT));
        mi.addActionListener(m_aeHandler);

        return menu;
    }

    /**
     * Creates and returns the Edit Sub menu. Call only once during init.
     * 
     * @return the created Edit Menu.
     */
    private JMenu makeEditMenu() {
        JMenuItem mi = null;

        JMenu menu = new JMenu("Edit");

        menu.add(mi = new JMenuItem(ActionEventsHandler.COPY));
        mi.addActionListener(m_aeHandler);

        return menu;
    }

    /**
     * Creates and returns the View Sub menu. Call only once during init. <br>
     * Create the View menu after creating any components controlled by this
     * menu, (like the toolbar) this way we can check their initial view and
     * create the menu accordingly.
     * 
     * @return the created View Menu.
     */
    private JMenu makeViewMenu() {
        JMenuItem mi = null;

        JMenu menu = new JMenu("View");

        menu.add(mi = new JCheckBoxMenuItem(ActionEventsHandler.SHOWTOOLBAR));
        mi.addActionListener(m_aeHandler);
        if (m_toolbar != null) {
            mi.setSelected(m_toolbar.isVisible());
        }

        menu.add(mi = new JCheckBoxMenuItem(ActionEventsHandler.TABONTOP));
        mi.addActionListener(m_aeHandler);
        if (m_tabs != null && m_tabs.getTabPlacement() == JTabbedPane.TOP) {
            mi.setSelected(true);
        }

        menu.add(mi = new JMenuItem(ActionEventsHandler.SHOWDESC));
        mi.addActionListener(m_aeHandler);

        return menu;
    }

    /**
     * Creates and returns the Log Sub menu. Call only once during init.
     * 
     * @return the created Log Menu.
     */
    private JMenu makeLogMenu() {
        JMenuItem mi = null;

        JMenu menu = new JMenu("Log");
        JMenu menuLvl = new JMenu("Level");

        //This menu requires a button group, as only one entry at a time will
        // be selected.
        ButtonGroup logMenuGroup = new ButtonGroup();

        menu.add(menuLvl);

        menuLvl.add(mi = new JRadioButtonMenuItem(ActionEventsHandler.LOGOFF));
        logMenuGroup.add(mi);
        mi.addActionListener(m_aeHandler);
        if (m_globalLogger.getLevel() == Level.OFF)
            mi.setSelected(true);

        menuLvl.add(mi = new JRadioButtonMenuItem(ActionEventsHandler.LOGALL));
        logMenuGroup.add(mi);
        mi.addActionListener(m_aeHandler);
        if (m_globalLogger.getLevel() == Level.ALL)
            mi.setSelected(true);

        menuLvl.add(mi = new JRadioButtonMenuItem(ActionEventsHandler.LOGINFO));
        logMenuGroup.add(mi);
        mi.addActionListener(m_aeHandler);
        if (m_globalLogger.getLevel() == Level.INFO)
            mi.setSelected(true);

        menuLvl.add(mi = new JRadioButtonMenuItem(ActionEventsHandler.LOGWARN));
        logMenuGroup.add(mi);
        mi.addActionListener(m_aeHandler);
        if (m_globalLogger.getLevel() == Level.WARNING)
            mi.setSelected(true);

        menuLvl
                .add(mi = new JRadioButtonMenuItem(ActionEventsHandler.LOGERROR));
        logMenuGroup.add(mi);
        mi.addActionListener(m_aeHandler);
        if (m_globalLogger.getLevel() == Level.SEVERE)
            mi.setSelected(true);

        menu.add(new JSeparator());
        menu.add(mi = new JMenuItem(ActionEventsHandler.CLEAR));
        mi.addActionListener(m_aeHandler);

        //TODO Add a debug level, that logs more debug formatted messages
        //menu.add(mi = new JCheckBoxMenuItem(ActionEventsHandler.LOGDEBUG));
        //mi.addActionListener(aeHandler);

        return menu;
    }

    /**
     * This is the main method which populates all views in the main window
     * based on the supplied search folder.
     * 
     * @param sBaseDir -
     *            The folder to start searching at. Not null.
     */
    public void populate(String sBaseDir) {
        m_taDescription.setText("Searching: " + sBaseDir + " ...");
        m_taDescriptionFull.setText("");

        BrwFileHunter brw = new BrwFileHunter(sBaseDir);

        DefaultTreeModel model = brw.getBrwModel();
        m_tree.setModel(model);

        m_taDescription.append("\nDone.");
    }

    /**
     * This method is used to populate information in the views based on the
     * currently selected application.
     */
    private void populateView() {
        if (m_currAppDesc != null) {
            m_globalLogger.log(Level.INFO, "Using descriptor file: "
                    + m_currAppDesc.getPath());
            SetupFileHunter s = new SetupFileHunter(m_currAppDesc);
            m_list.setListData(s.getFiles());
            m_taDescription.setText(m_currAppDesc.getShortDescriptionText());
            m_taDescriptionFull.setText(m_currAppDesc.getLongDescriptionText());

            //enable the applications folder path to be copied
            m_strSelected = new StringSelection(m_currAppDesc.getName()
                    + " ("
                    + GlobalUtils.toCanonicalPath(m_currAppDesc
                            .getContainingFolder()) + ")");
        }
    }

    /*
     * Handles the trees events
     * 
     * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
     */
    public void valueChanged(TreeSelectionEvent tev) {
        if (tev.getNewLeadSelectionPath() != null) {
            DefaultMutableTreeNode sel = (DefaultMutableTreeNode) tev
                    .getNewLeadSelectionPath().getLastPathComponent();

            //get the selected element
            Object selObj = sel.getUserObject();
            if (selObj instanceof IBrwFile) {
                //The user selected an application
                m_currAppDesc = (IBrwFile) selObj;
                populateView();
            } else {
                //the user clicked on a Category Heading
                String descStr = "Category: " + sel.toString();
                m_taDescription.setText(descStr);

                //enable the selection to be copied
                m_strSelected = new StringSelection(descStr);
            }
        }
    }

    /*
     * We are a LoggerComponent. Just forward any log messages to the Log Box.
     */
    public void log(String sMessage) {
        m_taLogBox.append(sMessage + "\n");
    }

    /*
     * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
     */
    public void focusGained(FocusEvent arg0) {
        // remember text for copy operations
        Component wndFocused = arg0.getComponent();
        if (wndFocused instanceof JTextArea) {
            m_strSelected = new StringSelection(((JTextArea) wndFocused)
                    .getText());
        }
        if (wndFocused instanceof JEditorPane) {
            m_strSelected = new StringSelection(((JEditorPane) wndFocused)
                    .getText());
        }
    }

    /*
     * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
     */
    public void focusLost(FocusEvent arg0) {
        //zzz...nothing to do
    }
}